# Improving on plain cd/pwd/glob

proc dirbrowser_initdir d {
    global dirbrowser
    set cmd [list cd $d]
    if $dirbrowser(usecd) {
	if [llength [array names dirbrowser proc]] {
	    if [catch {
		send $dirbrowser(sendid) $cmd
		set dirbrowser(usesend) 1
	    }] {
		puts $dirbrowser(proc) $cmd
		flush $dirbrowser(proc)
		set dirbrowser(usesend) 0
	    }
	} elseif {[catch {
	    if [catch {
		set dirbrowser(proc) [open "|[info nameofexecutable]" r+]
	    }] {
		set dirbrowser(proc) [open "|wish4.0" r+]
	    }
	    puts $dirbrowser(proc) {wm withdraw .}
	    puts $dirbrowser(proc) {puts stdout [tk appname]}
	    puts $dirbrowser(proc) {flush stdout}
	    flush $dirbrowser(proc)
	    # Kill at the appropriate time
	    rename exit __dirbrowser_exit
	    ;proc exit args {
		# The semicolon is to fool auto_mkindex
		global dirbrowser
		puts  $dirbrowser(proc) {exit}
		flush $dirbrowser(proc)
		close $dirbrowser(proc)
		eval __dirbrowser_exit $args
	    }
	    set dirbrowser(sendid) [gets $dirbrowser(proc)]
	    if [catch {
		send $dirbrowser(sendid) $cmd
		set dirbrowser(usesend) 1
	    }] {
		puts $dirbrowser(proc) $cmd
		flush $dirbrowser(proc)
		set dirbrowser(usesend) 0
	    }
	}]} {
	    set dirbrowser(usecd) 0
	}
    }
    if !$dirbrowser(usecd) {
	# This is not an else clause, since the above if can reset the flag
	set olddir [pwd]
	catch {
	    cd $d
	}
	set dirbrowser(cwd) "[pwd]/"
	cd $olddir
    }
}

proc dirbrowser_chdir d {
    global dirbrowser

    if $dirbrowser(usecd) {
	if $dirbrowser(usesend) {
	    send $dirbrowser(sendid) [list cd $d]
	} else {
	    puts $dirbrowser(proc) [list cd $d]
	    flush $dirbrowser(proc)
	}
    } else {
	# dirbrowser(cwd) is the current directory, with a trailing slash
	switch $d {
	    . {
		# This space intentionally left blank
	    }
	    .. {
		set dirbrowser(cwd) [file dirname \
			[file dirname $dirbrowser(cwd)]]/
		### Chop off an element if not at the root
		# if {"$dirbrowser(cwd)" != "/"} {
		#   regexp {^(.*/)[^/]*/$} $dirbrowser(cwd) foo dirbrowser(cwd)
		# }
	    }
	    default {
		# Go there!
		if {[info tclversion]==7.4} {
		    if {[string index $d 0] == "/"} {
			set dirbrowser(cwd) "$d/"
		    } else {
			append dirbrowser(cwd) "$d/"
		    }
		} else {
		    global tcl_platform
		    switch $tcl_platform(platform) {
			windows {
			    if [string match ?:* $d] {
				set dirbrowser(cwd) "$d/"
			    } elseif {[string index $d 0] == "/"} {
				set dirbrowser(cwd) \
					"[string index $dirbrowser(cwd) 0]:$d/"
			    } else {
				append dirbrowser(cwd) "$d/"
			    }

			}
			default {
			    if {[string index $d 0] == "/"} {
				set dirbrowser(cwd) "$d/"
			    } else {
				append dirbrowser(cwd) "$d/"
			    }
			}
		    }
		}
	    }
	}
    }
}

proc dirbrowser_pwd {} {
    global dirbrowser

    if $dirbrowser(usecd) {
	if $dirbrowser(usesend) {
	    return [send $dirbrowser(sendid) pwd]
	} else {
	    puts $dirbrowser(proc) {puts stdout [pwd]; flush stdout}
	    flush $dirbrowser(proc)
	    return [gets $dirbrowser(proc)]
	}
    } else {
	catch {
	    regexp {^((/*[^/]+)*)/+$} $dirbrowser(cwd) foo cwd
	    switch $tcl_platform(platform) {
		windows {
		    regexp {^(([a-z]:)?(/*[^/]+)*)/+$} $dirbrowser(cwd) foo cwd
		}
	    }
	}
	return $cwd
    }
}

# Searches for directories in CWD if no arg given
proc dirbrowser_glob {{pat {}}} {
    global dirbrowser

    switch -exact -- $pat {
	{} {
	    if $dirbrowser(usecd) {
		if $dirbrowser(usesend) {
		    return [glob */]
		} else {
		    # Quotes the glob for transfer across a pipe
		    puts $dirbrowser(proc) {
			puts stdout [list [glob -nocomplain */]]
			flush stdout
		    }
		    flush $dirbrowser(proc)
		    set glob [gets $dirbrowser(proc)]
		    while {![info complete $glob]} {
			append glob "\n" [gets $dirbrowser(proc)]
		    }
		    # Unquote the glob
		    return [lindex $glob 0]
		}
	    } else {
		return [glob [dirbrowser_pwd]/*/]
	    }
	}
	default {
	    if $dirbrowser(usecd) {
		if $dirbrowser(usesend) {
		    return [glob $pat]
		} else {
		    # Quotes the glob for transfer across a pipe
		    puts $dirbrowser(proc) [list set pat $pat]
		    puts $dirbrowser(proc) {
			puts stdout [list [glob -nocomplain $pat]]
			flush stdout
		    }
		    flush $dirbrowser(proc)
		    set glob [gets $dirbrowser(proc)]
		    while {![info complete $glob]} {
			append glob "\n" [gets $dirbrowser(proc)]
		    }
		    # Unquote the glob
		    return [lindex $glob 0]
		}
	    } else {
		return [glob $pat]
	    }
	}
    }
}

proc dirbrowser_getdirlist {directory} {
    global dirbrowser
    if [catch {set d1 [dirbrowser_glob $directory/*/]}] {
	set d1 {}
    }
    if {!$dirbrowser(hiddendirs) || \
	    [catch {set d2 [dirbrowser_glob $directory/.*/]}]} {
	set d2 {}
    }
    return [lsort [concat $d1 $d2]]
}

proc dirbrowser_getfilelist {directory filemask} {
    global dirbrowser dirbrowser_filename
    set files {}
    catch {
	foreach dirbrowserFilename [dirbrowser_glob $directory/$filemask] {
	    file stat $dirbrowserFilename filestats
	    set isadir [expr $filestats(mode) & 040000]
	    set dirbrowser_filename $dirbrowserFilename
	    if {!$isadir && [uplevel #0 expr $dirbrowser(filetest)]} {
		lappend files $dirbrowserFilename
	    }
	}
    }
    if $dirbrowser(hiddenfiles) {
	catch {
	    foreach dirbrowserFilename \
		    [dirbrowser_glob $directory/.$filemask] {
		file stat $dirbrowserFilename filestats
		set isadir [expr $filestats(mode) & 040000]
		set dirbrowser_filename $dirbrowserFilename
		if {!$isadir && [uplevel #0 expr $dirbrowser(filetest)]} {
		    lappend files $dirbrowserFilename
		}
	    }
	}
    }
    return [lsort $files]
}
